import api
import appModuleHandler
from ctypes import *
from ctypes.wintypes import *
import displayModel
from displayModel import EditableTextDisplayModelTextInfo
from NVDAObjects.IAccessible.delphi import Form
from NVDAObjects.window import Window,DisplayModelEditableText
import queueHandler
import speech
import textInfos
import winUser

def d(s):
	speech.speakMessage(s)
	
class TEditControlTextInfo(EditableTextDisplayModelTextInfo):
	def _getCaretOffset(self):
		caretRect = winUser.getGUIThreadInfo(self.obj.windowThreadID).rcCaret
		objLocation=self.obj.location
		objRect=RECT(objLocation[0],objLocation[1],objLocation[0]+objLocation[2],objLocation[1]+objLocation[3])
		tempPoint = winUser.POINT()
		tempPoint.x=caretRect.left
		tempPoint.y=caretRect.top
		winUser.user32.ClientToScreen(self.obj.windowHandle, byref(tempPoint))
		caretRect.left=max(objRect.left,tempPoint.x)
		caretRect.top=max(objRect.top,tempPoint.y)
		tempPoint.x=caretRect.right
		tempPoint.y=caretRect.bottom
		winUser.user32.ClientToScreen(self.obj.windowHandle, byref(tempPoint))
		caretRect.right=min(objRect.right,tempPoint.x)
		caretRect.bottom=min(objRect.bottom,tempPoint.y)
		caretRect.left+=1
		for charOffset, (charLeft, charTop, charRight, charBottom) in enumerate(self._textAndRects[1]):
			if caretRect.left>=charLeft and caretRect.right<=charRight and ((caretRect.top<=charTop and caretRect.bottom>=charBottom) or (caretRect.top>=charTop and caretRect.bottom<=charBottom)):
				return charOffset
		raise RuntimeError

class TMessageForm(Form):
	def _get_description(self):
		text=self.displayText.splitlines()[:-1]
		return "\n".join(text)

class TPropSelection(Window):
	def script_reportCurrentItem(self, gesture):
		gesture.send() #let arrows to do its work
		t=self.makeTextInfo(textInfos.POSITION_FIRST)
		t.expand(textInfos.UNIT_LINE)
		speech.speakTextInfo(t,unit=textInfos.UNIT_LINE,reason=speech.REASON_CARET)

	__gestures={
		"kb:upArrow":"reportCurrentItem",
		"kb:downArrow":"reportCurrentItem",
		"kb:pageUp":"reportCurrentItem",
		"kb:pageDown":"reportCurrentItem",
	}

class TypedCharactersMonitor:
	"""
	Use this class to speak typed characters when application support for this is broken.
	Just call handleChange in event_caret.
	"""

	def __init__(self, obj):
		self._obj=obj
		self._caretY=None
		self._line=None
		self._oldCaretY=None
		self._oldLine=None

	def handleChange(self):
		self._oldCaretY,self._oldLine=self._caretY,self._line
		self._caretY=winUser.getGUIThreadInfo(self._obj.windowThreadID).rcCaret.top
		t=self._obj.makeTextInfo(textInfos.POSITION_CARET)
		t.expand(textInfos.UNIT_LINE)
		self._line=t.text.replace("\0"," ")
		if self._caretY==None or self._caretY!=self._oldCaretY: return
		if len(self._line)-len(self._oldLine)>=1:
			t=self._obj.makeTextInfo(textInfos.POSITION_CARET)
			t.move(textInfos.UNIT_CHARACTER,-1)
			t.expand(textInfos.UNIT_CHARACTER)
			ch=t.text
			if ch==" ": speech.speakSpelling(" ") #Space is not spoken for some reason
			else: speech.speakTypedCharacters(ch)

class TEditControl(DisplayModelEditableText):
	TextInfo=TEditControlTextInfo

	def _caretScriptPostMovedHelper(self,speakUnit,info=None):
		self.redraw()
		super(TEditControl,self)._caretScriptPostMovedHelper(speakUnit,info)

	def event_gainFocus(self):
		self.redraw()
		if not hasattr(self,"_monitor"):
			self._monitor=TypedCharactersMonitor(self)
		self._monitor.handleChange()
	def event_caret(self):
		self._monitor.handleChange()
class AppModule(appModuleHandler.AppModule):
	def event_NVDAObject_init(self,obj):
		if obj.windowClassName in("Edit","TEdit","TPropEdit","TComboBox") and obj.name is None:
			try:
				t=obj.parent.flatReviewPosition
				t.expand(textInfos.UNIT_LINE)
			except:
				t=None
			if t is None: return
			name=t.text
			try:
				i=name.index(":")
			except:
				i=-1
			if i>=0: obj.name=name[:i+1]

	def chooseNVDAObjectOverlayClasses(self, obj, clsList):
		if obj.windowClassName=="TEditControl":
			clsList.insert(0,TEditControl)
		elif obj.windowClassName=="TMessageForm":
			clsList.insert(0,TMessageForm)
		elif obj.windowClassName=="TPropSelection":
			clsList.insert(0,TPropSelection)
